@@ -6,7 +6,6 @@
 Find warnings in byte code from Python source files.
 """
 
-import sys
 import string
 import types
 
@@ -15,6 +14,7 @@ from pychecker import utils
 from pychecker import Warning
 from pychecker import OP
 from pychecker import Stack
+from pychecker import python
 
 __pychecker__ = 'no-argsused'
 
@@ -43,7 +43,8 @@ def _checkFunctionArgCount(code, func_name, argCount, minArgs, maxArgs,
     if err :
         code.addWarning(err)
 
-def _checkFunctionArgs(code, func, objectReference, argCount, kwArgs) :
+def _checkFunctionArgs(code, func, objectReference, argCount, kwArgs,
+                       check_arg_count = 1) :
     func_name = func.function.func_code.co_name
     if kwArgs :
         func_args = func.function.func_code.co_varnames
@@ -58,14 +59,16 @@ def _checkFunctionArgs(code, func, objectReference, argCount, kwArgs) :
                   kwArgs[0] in func_args[origArgCount:] :
                 argCount = argCount + 1
                 kwArgs = kwArgs[1:]
-            _checkFunctionArgs(code, func, objectReference, argCount, kwArgs)
+            _checkFunctionArgs(code, func, objectReference, argCount, kwArgs,
+                               check_arg_count)
             return
 
         if not func.supportsKW :
             code.addWarning(msgs.FUNC_DOESNT_SUPPORT_KW % func_name)
 
-    _checkFunctionArgCount(code, func_name, argCount,
-                           func.minArgs, func.maxArgs, objectReference)
+    if check_arg_count :
+        _checkFunctionArgCount(code, func_name, argCount,
+                               func.minArgs, func.maxArgs, objectReference)
 
 def _getReferenceFromModule(module, identifier) :
     func = module.functions.get(identifier, None)
@@ -116,70 +119,15 @@ def _getFunction(module, stackValue) :
         return None, None, 0
     return c.methods.get(identifier[-1], None), c, 0
 
-_BOOL = types.IntType
-#                     name   (type,  args: min, max, kwArgs?
-_GLOBAL_FUNC_INFO = { 'abs': (Stack.TYPE_UNKNOWN, 1, 1),
-                      'buffer': (types.BufferType, 1, 3),
-                      'callable': (_BOOL, 1, 1),
-                      'chr': (types.StringType, 1, 1),
-                      'cmp': (_BOOL, 2, 2),
-                      'coerce': ([ types.NoneType, types.TupleType ], 2, 2),
-                      'compile': (types.CodeType, 3, 3),
-                      'complex': (types.ComplexType, 1, 2),
-                      'delattr': (types.NoneType, 2, 2),
-                      'dir': (types.ListType, 0, 1),
-                      'divmod': (types.TupleType, 2, 2),
-                      'eval': (Stack.TYPE_UNKNOWN, 1, 3),
-                      'execfile': (types.NoneType, 1, 3),
-                      'filter': (types.ListType, 2, 2),
-                      'float': (types.FloatType, 1, 1),
-                      'getattr': (Stack.TYPE_UNKNOWN, 2, 3),
-                      'globals': (types.DictType, 0, 0),
-                      'hasattr': (_BOOL, 2, 2),
-                      'hash': (types.IntType, 1, 1),
-                      'hex': (types.StringType, 1, 1),
-                      'id': (types.IntType, 1, 1),
-                      'int': (types.IntType, 1, 2),
-                      'intern': (types.StringType, 1, 1),
-                      'isinstance': (_BOOL, 2, 2),
-                      'issubclass': (_BOOL, 2, 2),
-                      'len': (types.IntType, 1, 1),
-                      'list': (types.ListType, 1, 1),
-                      'locals': (types.DictType, 0, 0),
-                      'long': (types.LongType, 1, 2),
-                      'map': (types.ListType, 2, None),
-                      'max': (Stack.TYPE_UNKNOWN, 1, None),
-                      'min': (Stack.TYPE_UNKNOWN, 1, None),
-                      'oct': (types.StringType, 1, 1),
-                      'open': (types.FileType, 1, 3),
-                      'ord': (types.IntType, 1, 1),
-                      'pow': (Stack.TYPE_UNKNOWN, 2, 3),
-                      'range': (types.ListType, 1, 3),
-                      'reduce': (Stack.TYPE_UNKNOWN, 2, 3),
-                      'reload': (types.ModuleType, 1, 1),
-                      'repr': (types.StringType, 1, 1),
-                      'round': (types.FloatType, 1, 2),
-                      'setattr': (types.NoneType, 3, 3),
-                      'slice': (types.SliceType, 1, 3),
-                      'str': (types.StringType, 1, 1),
-                      'tuple': (types.TupleType, 1, 1),
-                      'type': (types.TypeType, 1, 1),
-                      'unichr': (types.StringType, 1, 1), # FIXME: unicode
-                      'unicode': (types.StringType, 1, 3), # FIXME: unicode
-                      'vars': (types.DictType, 0, 1),
-                      'xrange': (types.ListType, 1, 3),
-                      'zip': (types.ListType, 1, None),
-                    }
-
-def _checkBuiltin(code, loadValue, argCount, kwArgs) :
+def _checkBuiltin(code, loadValue, argCount, kwArgs, check_arg_count = 1) :
     returnValue = Stack.makeFuncReturnValue(loadValue)
     func_name = loadValue.data
     if loadValue.type == Stack.TYPE_GLOBAL :
-        info = _GLOBAL_FUNC_INFO.get(func_name, None)
+        info = python.GLOBAL_FUNC_INFO.get(func_name, None)
         if info is not None :
             if kwArgs :
                 code.addWarning(msgs.FUNC_DOESNT_SUPPORT_KW % func_name)
-            else :
+            elif check_arg_count :
                 _checkFunctionArgCount(code, func_name, argCount,
                                        info[1], info[2])
             returnValue = Stack.Item(func_name, info[0])
@@ -189,12 +137,48 @@ def _checkBuiltin(code, loadValue, argCount, kwArgs) :
             try :
                 if func_name[1] == 'append' and argCount > 1 :
                     code.addWarning(msgs.LIST_APPEND_ARGS % func_name[0])
+                    check_arg_count = 0
             except AttributeError :
+                # FIXME: why do we need to catch AttributeError???
                 pass
+        if len(objType) == 1 :
+            # if it's a builtin, check method
+            builtinType = python.BUILTIN_METHODS.get(objType[0])
+            if builtinType is not None :
+                methodInfo = builtinType.get(func_name[1])
+                # set func properly
+                if kwArgs :
+                    code.addWarning(msgs.FUNC_DOESNT_SUPPORT_KW % func_name[1])
+                elif methodInfo :
+                    returnValue = Stack.Item(func_name[1], methodInfo[0])
+                    if check_arg_count and methodInfo is not None :
+                        _checkFunctionArgCount(code, func_name[1], argCount,
+                                               methodInfo[1], methodInfo[2])
 
     return returnValue
 
-def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0) :
+def _checkModifyDefaultArg(code, objectName) :
+    try :
+        value = code.func.defaultValue(objectName)
+        if type(value) in python.MUTABLE_TYPES :
+            code.addWarning(msgs.MODIFYING_DEFAULT_ARG % objectName)
+    except ValueError :
+        pass
+
+def _isexception(object) :
+    # FIXME: i have no idea why this function is necessary
+    # it seems that the issubclass() should work, but it doesn't always
+    if issubclass(object, Exception) :
+        return 1
+    for c in object.__bases__ :
+        if utils.startswith(str(c), 'exceptions.') :
+            return 1
+        if len(c.__bases__) > 0 and _isexception(c) :
+            return 1
+    return 0
+
+def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
+                        check_arg_count = 1) :
     'Checks for warnings, returns function called (may be None)'
 
     if not code.stack :
@@ -225,32 +209,61 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0) :
         try :
             m = codeSource.classObject.methods[methodName]
             if m != None :
-                _checkFunctionArgs(code, m, 1, argCount, kwArgs)
+                _checkFunctionArgs(code, m, 1, argCount, kwArgs, check_arg_count)
         except KeyError :
             if cfg().callingAttribute :
                 code.addWarning(msgs.INVALID_METHOD % methodName)
-    elif loadValue.type in [ Stack.TYPE_ATTRIBUTE, Stack.TYPE_GLOBAL, ] and \
-         type(loadValue.data) in [ types.StringType, types.TupleType ] :
+    elif loadValue.type in (Stack.TYPE_ATTRIBUTE, Stack.TYPE_GLOBAL) and \
+         type(loadValue.data) in (types.StringType, types.TupleType) :
         # apply(func, (args)), can't check # of args, so just return func
         if loadValue.data == 'apply' :
             loadValue = code.stack[funcIndex+1]
         else :
-            func, refClass, create = _getFunction(codeSource.module, loadValue)
+            if cfg().modifyDefaultValue and \
+               type(loadValue.data) == types.TupleType :
+                _checkModifyDefaultArg(code, loadValue.data[0])
+
+            func, refClass, method = _getFunction(codeSource.module, loadValue)
+            if func == None and type(loadValue.data) == types.TupleType and \
+               len(loadValue.data) == 2 :
+                # looks like we are making a method call
+                data = loadValue.data
+                if type(data[0]) == types.StringType :
+                    # do we know the type of the local variable?
+                    varType = code.typeMap.get(data[0])
+                    if varType is not None and len(varType) == 1 :
+                        if hasattr(varType[0], 'methods') :
+                            # it's a class & we know the type, get the method
+                            func = varType[0].methods.get(data[1])
+                            if func is not None :
+                                method = 1
+
             if func != None :
-                _checkFunctionArgs(code, func, create, argCount, kwArgs)
-                if refClass and argCount > 0 and not create and \
-                   code.stack[funcIndex].type == Stack.TYPE_ATTRIBUTE and \
-                   code.stack[funcIndex+1].data != cfg().methodArgName :
-                    code.addWarning(msgs.SELF_NOT_FIRST_ARG % cfg().methodArgName)
-            elif refClass and create :
+                _checkFunctionArgs(code, func, method, argCount, kwArgs, check_arg_count)
+                if refClass :
+                    if method :
+                        # c'tor, return the class as the type
+                        returnValue = Stack.Item(loadValue, refClass)
+                    elif argCount > 0 and \
+                         code.stack[funcIndex].type == Stack.TYPE_ATTRIBUTE and \
+                         code.stack[funcIndex+1].data != cfg().methodArgName :
+                        code.addWarning(msgs.SELF_NOT_FIRST_ARG % cfg().methodArgName)
+            elif refClass and method :
                 returnValue = Stack.Item(loadValue, refClass)
                 if (argCount > 0 or len(kwArgs) > 0) and \
                    not refClass.ignoreAttrs and \
                    not refClass.methods.has_key(utils.INIT) and \
-                   not issubclass(refClass.classObject, Exception) :
+                   not _isexception(refClass.classObject) :
                     code.addWarning(msgs.NO_CTOR_ARGS)
             else :
-                returnValue = _checkBuiltin(code, loadValue, argCount, kwArgs)
+                returnValue = _checkBuiltin(code, loadValue, argCount, kwArgs,
+                                            check_arg_count)
+                if returnValue.type is types.NoneType and \
+                   not OP.POP_TOP(code.nextOpInfo()[0]) :
+                    name = str(loadValue.data)
+                    if type(loadValue.data) == types.TupleType :
+                        name = string.join(loadValue.data, '.')
+                    code.addWarning(msgs.USING_NONE_RETURN_VALUE % name)
 
     code.stack = code.stack[:funcIndex] + [ returnValue ]
     if loadValue is not None :
@@ -355,7 +368,7 @@ def _handleImport(code, operand, module, main, fromName) :
                 err = msgs.MODULE_MEMBER_ALSO_STAR_IMPORTED % fromName
 
         # filter out warnings when files are different (ie, from X import ...)
-        if err :
+        if err is not None and cfg().moduleImportErrors :
             bytes = module.main_code
             if bytes is None or \
                bytes.function.func_code.co_filename == code.func_code.co_filename :
@@ -390,6 +403,7 @@ def _getFormatInfo(format, code) :
 
     # skip the first item in the list, it's always empty
     for section in sections[1:] :
+        orig_section = section
         # handle dictionary formats
         if section[0] == '(' :
             mappingFormatCount = mappingFormatCount + 1
@@ -420,7 +434,7 @@ def _getFormatInfo(format, code) :
 
         formatCount = formatCount + stars
         if section[i] not in _FORMAT_CONVERTERS :
-            code.addWarning(msgs.INVALID_FORMAT % section)
+            code.addWarning(msgs.INVALID_FORMAT % orig_section)
 
     if mappingFormatCount > 0 and mappingFormatCount != percentFormatCount :
         code.addWarning(msgs.CANT_MIX_MAPPING_IN_FORMATS)
@@ -439,9 +453,7 @@ def _getConstant(code, module, data) :
     return None
 
 _UNCHECKABLE_FORMAT_STACK_TYPES = \
-    [
-      Stack.TYPE_UNKNOWN, Stack.TYPE_FUNC_RETURN, Stack.TYPE_ATTRIBUTE,
-    ]
+      (Stack.TYPE_UNKNOWN, Stack.TYPE_FUNC_RETURN, Stack.TYPE_ATTRIBUTE,)
 
 def _getFormatWarnings(code, codeSource) :
     if len(code.stack) <= 1 :
@@ -465,120 +477,51 @@ def _getFormatWarnings(code, codeSource) :
             else :
                 code.unusedLocals[varname] = None
     else :
-        if ((topOfStack.type == types.DictType and len(vars) > 0) or
+        stackItemType = topOfStack.getType(code.typeMap)
+        if ((stackItemType == types.DictType and len(vars) > 0) or
             codeSource.func.isParam(topOfStack.data) or
-            topOfStack.type in _UNCHECKABLE_FORMAT_STACK_TYPES) :
+            stackItemType in _UNCHECKABLE_FORMAT_STACK_TYPES) :
             return
 
         if topOfStack.type == types.TupleType :
             args = topOfStack.length
+        elif stackItemType == types.TupleType :
+            args = len(code.constants.get(topOfStack.data, (0,)))
         else :
             args = 1
-            # if we have a variable reference
-            if topOfStack.type == types.StringType and not topOfStack.const :
-                # and if the type is a tuple, get the length
-                dataTypes = code.typeMap.get(topOfStack.data, [])
-                if types.TupleType in dataTypes :
-                    args = 0
-                    if len(dataTypes) == 1 :
-                        args = len(code.constants.get(topOfStack.data, (0,)))
 
     if args and count != args :
         code.addWarning(msgs.INVALID_FORMAT_COUNT % (count, args))
 
-
-_METHODLESS_OBJECTS = { types.NoneType : None, types.IntType : None,
-                        types.LongType : None, types.FloatType : None,
-                        types.BufferType : None, types.TupleType : None,
-                        types.EllipsisType : None,
-                      }
-
-_BUILTINS_ATTRS = { types.StringType : dir(''),
-                    types.TypeType : dir(type(0)),
-                    types.ListType : dir([]),
-                    types.DictType : dir({}),
-                    types.FunctionType : dir(cfg),
-                    types.BuiltinFunctionType : dir(len),
-                    types.BuiltinMethodType : dir([].append),
-                    types.ClassType : dir(Stack.Item),
-                    types.UnboundMethodType : dir(Stack.Item.__init__),
-                    types.LambdaType : dir(lambda: None),
-                    types.SliceType : dir(slice(0)),
-                  }
-
-def _setupBuiltinAttrs() :
-    w = Warning.Warning('', 0, '')
-    _BUILTINS_ATTRS[types.MethodType] = dir(w.__init__)
-    del w
-
-    if utils.pythonVersion() >= utils.PYTHON_2_2 :
-        # FIXME: I'm sure more types need to be added here
-        _BUILTINS_ATTRS[types.StringType] = dir(''.__class__)
-        _BUILTINS_ATTRS[types.ListType] = dir([].__class__)
-        _BUILTINS_ATTRS[types.DictType] = dir({}.__class__)
-
-    try :
-        import warnings
-        _MSG = "xrange object's 'start', 'stop' and 'step' attributes are deprecated"
-        warnings.filterwarnings('ignore', _MSG)
-        del warnings, _MSG
-    except ImportError :
-        pass
-    _BUILTINS_ATTRS[types.XRangeType] = dir(xrange(0))
-
-    try: _BUILTINS_ATTRS[types.ComplexType] = dir(complex(0, 1))
-    except: pass
-
-    try: _BUILTINS_ATTRS[types.UnicodeType] = dir(unicode(''))
-    except: pass
-
-    try: _BUILTINS_ATTRS[types.CodeType] = dir(_setupBuiltinAttrs.func_code)
-    except: pass
-
-    try: _BUILTINS_ATTRS[types.FileType] = dir(sys.__stdin__)
-    except: pass
-
-    try:
-        raise TypeError
-    except TypeError :
-        try:
-            tb = sys.exc_info()[2]
-            _BUILTINS_ATTRS[types.TracebackType] = dir(tb)
-            _BUILTINS_ATTRS[types.FrameType] = dir(tb.tb_frame)
-        except:
-            pass
-        tb = None; del tb
-
-# have to setup the rest this way to support different versions of Python
-_setupBuiltinAttrs()
-
 def _checkAttributeType(code, stackValue, attr) :
     if not cfg().checkObjectAttrs :
-        return None
+        return
 
     varTypes = code.typeMap.get(str(stackValue.data), None)
     if not varTypes :
-        return None
+        return
 
-    for varType in varTypes :
+    # the value may have been converted on stack (`v`)
+    other_types = []
+    if stackValue.type not in varTypes :
+        other_types = [stackValue.type]
+
+    for varType in varTypes + other_types :
         # ignore built-in types that have no attributes
-        if _METHODLESS_OBJECTS.has_key(varType) :
+        if python.METHODLESS_OBJECTS.has_key(varType) :
             continue
 
-        if type(varType) == types.StringType :
-            return None
-
-        attrs = _BUILTINS_ATTRS.get(varType, None)
+        attrs = python.BUILTIN_ATTRS.get(varType, None)
         if attrs is not None :
             if attr in attrs :
-                return None
+                return
             continue
 
         if hasattr(varType, 'ignoreAttrs') :
             if varType.ignoreAttrs or _classHasAttribute(varType, attr) :
-                return None
+                return
         elif not hasattr(varType, 'attributes') or attr in varType.attributes :
-            return None
+            return
 
     code.addWarning(msgs.OBJECT_HAS_NO_ATTR % (stackValue.data, attr))
 
@@ -588,6 +531,7 @@ class Code :
 
     def __init__(self) :
         self.bytes = None
+        self.func = None
         self.func_code = None
         self.index = 0
         self.extended_arg = 0
@@ -616,6 +560,7 @@ class Code :
         self.codeObjects = {}
 
     def init(self, func) :
+        self.func = func
         self.func_code, self.bytes, self.index, self.maxCode, self.extended_arg = \
                         OP.initFuncCode(func.function)
         self.lastLineNum = self.func_code.co_firstlineno
@@ -752,8 +697,14 @@ def _checkException(code, name) :
         if __builtins__.has_key(name) :
             code.addWarning(msgs.SET_EXCEPT_TO_BUILTIN % name)
 
+def _checkFutureKeywords(code, varname) :
+    kw = python.FUTURE_KEYWORDS.get(varname)
+    if kw is not None :
+        code.addWarning(msgs.USING_KEYWORD % (varname, kw))
+
 def _STORE_NAME(oparg, operand, codeSource, code) :
     if not code.updateCheckerArgs(operand) :
+        _checkFutureKeywords(code, operand)
         module = codeSource.module
         if not codeSource.in_class :
             _checkGlobal(operand, module, codeSource.func, code,
@@ -774,10 +725,12 @@ def _STORE_NAME(oparg, operand, codeSource, code) :
 _STORE_GLOBAL = _STORE_NAME
 
 def _checkLoadGlobal(codeSource, code, varname) :
+    _checkFutureKeywords(code, varname)
     should_check = 1
     if code.func_code.co_name == utils.LAMBDA :
         # this could really be a local reference, check first
         if not codeSource.main and \
+           hasattr(codeSource.calling_code, 'function') and \
            varname in codeSource.calling_code.function.func_code.co_varnames :
             _handleLoadLocal(code, codeSource.func, varname)
             should_check = 0
@@ -798,11 +751,21 @@ def _LOAD_NAME(oparg, operand, codeSource, code) :
         operand = eval("codeSource.module.module.%s.__name__" % operand)
 
     opType, const = Stack.TYPE_GLOBAL, 0
-    if operand in [ 'None', 'Ellipsis', ] :
-        opType, const = type(operand), 1
+    if operand == 'None' :
+        opType, const = types.NoneType, 0
+    elif operand == 'Ellipsis' :
+        opType, const = types.EllipsisType, 1
     code.stack.append(Stack.Item(operand, opType, const))
 
-_LOAD_GLOBAL = _LOAD_DEREF = _LOAD_NAME
+_LOAD_GLOBAL = _LOAD_NAME
+
+def _LOAD_DEREF(oparg, operand, codeSource, code) :
+    if type(oparg) == types.IntType :
+        argused = code.func_code.co_varnames[oparg]
+        code.unusedLocals[argused] = None
+    else :
+        _LOAD_GLOBAL(oparg, operand, codeSource, code)
+
 
 def _DELETE_NAME(oparg, operand, codeSource, code) :
     _checkLoadGlobal(codeSource, code, operand)
@@ -824,6 +787,7 @@ def _LOAD_CONST(oparg, operand, codeSource, code) :
 
 
 def _checkLoadLocal(code, func, varname, deletedWarn, usedBeforeSetWarn) :
+    _checkFutureKeywords(code, varname)
     deletedLine = code.deletedLocals.get(varname)
     if deletedLine :
         code.addWarning(deletedWarn % (varname, deletedLine))
@@ -841,6 +805,7 @@ def _LOAD_FAST(oparg, operand, codeSource, code) :
 
 def _STORE_FAST(oparg, operand, codeSource, code) :
     if not code.updateCheckerArgs(operand) :
+        _checkFutureKeywords(code, operand)
         code.setType(operand)
         if not code.unpackCount and code.stack and \
            (code.stack[-1].const or code.stack[-1].type == types.TupleType) :
@@ -850,6 +815,8 @@ def _STORE_FAST(oparg, operand, codeSource, code) :
                 code.constants[operand] = code.stack[-1].data
 
         _checkException(code, operand)
+        if code.deletedLocals.has_key(operand) :
+            del code.deletedLocals[operand]
         if not code.unusedLocals.has_key(operand) :
             errLine = code.lastLineNum
             if code.unpackCount and not cfg().unusedLocalTuple :
@@ -871,13 +838,34 @@ def _checkAttribute(top, operand, codeSource, code) :
     else :
         _checkAttributeType(code, top, operand)
 
+def _checkExcessiveReferences(code, top, extraAttr = None) :
+    if cfg().maxReferences <= 0 :
+        return
+
+    try :
+        data = top.data
+        if extraAttr is not None :
+            data = data + (extraAttr,)
+        
+        maxReferences = cfg().maxReferences
+        if data[0] == cfg().methodArgName :
+            maxReferences = maxReferences + 1
+        if len(data) > maxReferences :
+            name = string.join(top.data, '.')
+            code.addWarning(msgs.TOO_MANY_REFERENCES % (maxReferences, name))
+    except TypeError :
+        pass
+
 def _LOAD_ATTR(oparg, operand, codeSource, code) :
     if len(code.stack) > 0 :
         top = code.stack[-1]
         _checkAttribute(top, operand, codeSource, code)
         top.addAttribute(operand)
+        if not OP.LOAD_ATTR(code.nextOpInfo()[0]) :
+            _checkExcessiveReferences(code, top)
 
 def _STORE_ATTR(oparg, operand, codeSource, code) :
+    _checkExcessiveReferences(code, code.stack[-1], operand)
     code.unpack()
 
 def _DELETE_ATTR(oparg, operand, codeSource, code) :
@@ -914,6 +902,11 @@ def _DUP_TOP(oparg, operand, codeSource, code) :
         code.stack.append(code.stack[-1])
 
 def _pop2(oparg, operand, codeSource, code) :
+    if len(code.stack) >= 2 :
+        loadValue = code.stack[-2]
+        if cfg().modifyDefaultValue and loadValue.type == types.StringType :
+            _checkModifyDefaultArg(code, loadValue.data)
+
     code.popStackItems(2)
 _STORE_SUBSCR = _DELETE_SUBSCR = _pop2
 
@@ -921,13 +914,13 @@ def _CALL_FUNCTION(oparg, operand, codeSource, code) :
     _handleFunctionCall(codeSource, code, oparg)
 
 def _CALL_FUNCTION_VAR(oparg, operand, codeSource, code) :
-    _handleFunctionCall(codeSource, code, oparg, 1)
+    _handleFunctionCall(codeSource, code, oparg, 1, 0)
 
 def _CALL_FUNCTION_KW(oparg, operand, codeSource, code) :
     _handleFunctionCall(codeSource, code, oparg, 1)
 
 def _CALL_FUNCTION_VAR_KW(oparg, operand, codeSource, code) :
-    _handleFunctionCall(codeSource, code, oparg, 2)
+    _handleFunctionCall(codeSource, code, oparg, 2, 0)
 
 def _MAKE_FUNCTION(oparg, operand, codeSource, code) :
     code.popStackItems(oparg+1)
@@ -941,12 +934,20 @@ def _BUILD_LIST(oparg, operand, codeSource, code) :
 
 def _UNARY_CONVERT(oparg, operand, codeSource, code) :
     if len(code.stack) > 0 :
-        code.stack[-1].data = str(code.stack[-1].data)
-        code.stack[-1].type = types.StringType
+        stackValue = code.stack[-1]
+        if stackValue.data == cfg().methodArgName and \
+           stackValue.const == 0 and codeSource.classObject is not None and \
+           codeSource.func.function.func_name == '__repr__' :
+            code.addWarning(msgs.USING_SELF_IN_REPR)
+        stackValue.data = str(stackValue.data)
+        stackValue.type = types.StringType
 
 def _UNARY_POSITIVE(oparg, operand, codeSource, code) :
     if OP.UNARY_POSITIVE(code.nextOpInfo()[0]) :
         code.addWarning(msgs.STMT_WITH_NO_EFFECT % '++')
+        code.getNextOp()
+    elif cfg().unaryPositive and code.stack and not code.stack[-1].const :
+        code.addWarning(msgs.UNARY_POSITIVE_HAS_NO_EFFECT)
 
 def _UNARY_NEGATIVE(oparg, operand, codeSource, code) :
     if OP.UNARY_NEGATIVE(code.nextOpInfo()[0]) :
@@ -1108,8 +1109,7 @@ DISPATCH[126] = _DELETE_FAST
 DISPATCH[127] = _LINE_NUM
 DISPATCH[131] = _CALL_FUNCTION
 DISPATCH[132] = _MAKE_FUNCTION
+DISPATCH[136] = _LOAD_DEREF
 DISPATCH[140] = _CALL_FUNCTION_VAR
 DISPATCH[141] = _CALL_FUNCTION_KW
 DISPATCH[142] = _CALL_FUNCTION_VAR_KW
-
-
