@@ -54,7 +54,8 @@ def _implicitReturnUnreachable(code) :
 
 
 _IGNORE_RETURN_TYPES = ( Stack.TYPE_FUNC_RETURN, Stack.TYPE_ATTRIBUTE,
-                         Stack.TYPE_GLOBAL )
+                         Stack.TYPE_GLOBAL, Stack.TYPE_COMPARISON,
+                         Stack.TYPE_UNKNOWN)
 
 def _checkReturnWarnings(code) :
     # there must be at least 2 real return values to check for consistency
@@ -78,17 +79,17 @@ def _checkReturnWarnings(code) :
     returnType, returnData = None, None
     for line, value in code.returnValues :
         if not value.isNone() :
+            valueType = value.getType(code.typeMap)
             if returnType is None :
                 returnData = value
-                returnType = type(value.data)
+                returnType = valueType
 
             # always ignore None, None can be returned w/any other type
             # FIXME: if we stored func return values, we could do better
             if returnType is not None and not value.isNone() and \
-               not value.const and not returnData.const and \
-               value.type not in _IGNORE_RETURN_TYPES and \
+               valueType not in _IGNORE_RETURN_TYPES and \
                returnData.type not in _IGNORE_RETURN_TYPES :
-                ok = (returnType == type(value.data))
+                ok = returnType in (type(value.data), valueType)
                 if ok and returnType == types.TupleType :
                     ok = returnData.length == value.length
                 if not ok :
@@ -195,7 +196,7 @@ def _checkFunction(module, func, c = None, main = 0, in_class = 0) :
     _checkComplex(code, cfg().maxReturns, returns, func, msgs.TOO_MANY_RETURNS)
     _checkComplex(code, cfg().maxBranches, branches, func, msgs.TOO_MANY_BRANCHES)
 
-    if not main :
+    if not (main or in_class) :
         utils.popConfig()
     return (code.warnings, code.globalRefs, code.functionsCalled,
             code.codeObjects.values(), code.returnValues)
@@ -415,7 +416,13 @@ def _findClassWarnings(module, c, class_code,
         #        not a base class file???
         err = msgs.NO_CLASS_DOC % c.classObject.__name__
         warnings.append(Warning(filename, func_code, err))
-                                
+
+    # we have to do this here, b/c checkFunction doesn't popConfig for classes
+    # this allows us to have __pychecker__ apply to all methods
+    # when defined at class scope
+    if class_code is not None :
+        utils.popConfig()
+
     if classSuppress is not None :
         utils.popConfig()
 
